/*  A variation on Hume & Sunday's Tuned Boyer-Moore algorithm for
 *  searching for a literal string pattern in a text.
 *
 *  References:
 *    Hume, A., D.M. Sunday. Fast String Searching. _Software--Practice
 *      and Experience_. Vol. 21, No. 11, p. 1221-48. November 1991.
 *
 *  Copyright (c) 1993 Christopher J. Kane. All rights reserved.
 *
 *  This software is subject to the terms of the MiscKit license
 *  agreement. Refer to the license document included with the
 *  MiscKit distribution for these terms.
 *
 *  Version 1.0 (October 10, 1993)
 */

#include <misckit/MiscTBMK.h>
#import <ctype.h>
#import <mach/mach.h>
extern void *calloc(unsigned int, unsigned int);
extern void free(void *);

#define TBMKMAGIC 0x54424D4B

struct Misc_tbmk_pattern {
  unsigned int magic;
  unsigned char *pattern;
  unsigned int pattern_length;
  unsigned int skip[256];
  unsigned int jump;
  unsigned char reverse;
  unsigned char nocase;
};

#define uncase(C) ({int _c=(C); isupper(_c) ? _c-'A'+'a' : _c;})

Misc_TBMKpattern Misc_TBMKpattern_alloc(const unsigned char *pattern, unsigned int pattern_length, unsigned char reverse, unsigned char nocases)
{
  Misc_TBMKpattern new;
  int i;

  if (pattern_length==0)
    return 0;
  new = (Misc_TBMKpattern)calloc(1, sizeof(struct Misc_tbmk_pattern));
  if (new==0)
    return 0;
  new->magic = TBMKMAGIC;
  new->pattern = calloc(pattern_length+1, 1);
  if (new->pattern==0)
    {
      free(new);
      return 0;
    }
  memcpy(new->pattern, pattern, pattern_length);
  new->pattern[pattern_length] = '\0';
  new->pattern_length = pattern_length;
  if (nocases)
    for (i=pattern_length; i--;)
      new->pattern[i] = uncase(new->pattern[i]);
  for (i=256; i--;)
    new->skip[i] = pattern_length;
  pattern_length--;
  if (reverse)
    {
      for (i=pattern_length; i>=0; i--)
        new->skip[new->pattern[i]] = i;
      for (i=1; i<=pattern_length && new->pattern[i]!=new->pattern[0]; i++);
      new->jump = -i;
    }
  else
    {
      for (i=0; i<=pattern_length; i++)
        new->skip[new->pattern[i]] = pattern_length-i;
      for (i=pattern_length-1; i>=0 && new->pattern[i]!=new->pattern[pattern_length]; i--);
      new->jump = pattern_length-i;
    }
  new->reverse = reverse;
  new->nocase = nocases;
  return new;
}

void Misc_TBMKpattern_free(Misc_TBMKpattern *pattern)
{
  if (pattern!=0 && (*pattern)->magic==TBMKMAGIC)
    {
      if ((*pattern)->pattern!=0)
        free((*pattern)->pattern);
      free(*pattern);
      *pattern = 0;
    }
}

int Misc_TBMKsearch(const Misc_TBMKpattern pattern, const unsigned char *text, int text_extent, unsigned char all_matches, int *match_position)
{
  const unsigned char *s, *e, *tmpp;
  register int i, patlen, nmatch=0;

  if (!(pattern->magic==TBMKMAGIC
      && ((pattern->reverse && text_extent<0)
          || (!pattern->reverse && text_extent>0))))
    return -1;
  patlen = pattern->pattern_length-1;
  s = text+(pattern->reverse?-patlen:patlen);
  e = text+text_extent-(pattern->reverse?-1:1);
  while (1)
    {
      i = 0;
      if (pattern->nocase)
        {
          if (pattern->reverse)
            do if ((s -= i)<e) goto end; while (i = pattern->skip[uncase(*s)]);
          else
            do if ((s += i)>e) goto end; while (i = pattern->skip[uncase(*s)]);
          tmpp = pattern->reverse?s:s-patlen;
          while (i<=patlen && uncase(*tmpp++)==pattern->pattern[i++]);
        }
      else
        {
          if (pattern->reverse)
            do if ((s -= i)<e) goto end; while (i = pattern->skip[*s]);
          else
            do if ((s += i)>e) goto end; while (i = pattern->skip[*s]);
          tmpp = pattern->reverse?s:s-patlen;
          while (i<=patlen && *tmpp++==pattern->pattern[i++]);
        }
      if (i<=patlen)
        s += pattern->jump;
      else
        {
          nmatch++;
          *match_position = tmpp-text-pattern->pattern_length;
          if (!all_matches) goto end;
          s += (pattern->reverse?-1:1)*pattern->pattern_length;
        }
    }
 end:
  return nmatch;
}
