/*  A variation on Hume & Sunday's Tuned Boyer-Moore algorithm for
 *  searching for a literal string pattern in a text.
 *
 *  References:
 *    Hume, A., D.M. Sunday. Fast String Searching. _Software--Practice
 *      and Experience_. Vol. 21, No. 11, p. 1221-48. November 1991.
 *
 *  Copyright (c) 1993 Christopher J. Kane. All rights reserved.
 *
 *  This software is subject to the terms of the MiscKit license
 *  agreement. Refer to the license document included with the
 *  MiscKit distribution for these terms.
 *
 *  Version 1.0 (October 10, 1993)
 */

#ifndef _MISC_TBMK_H
#define _MISC_TBMK_H

/* The type Misc_TBMKpattern is a pointer to a structure used to hold
 * a compiled pattern. Create a compiled pattern by using the function
 * Misc_TBMKpattern_alloc(), destroy it with Misc_TBMKpattern_free().
 * The members of the structure should never be accessed directly. */

typedef struct Misc_tbmk_pattern * Misc_TBMKpattern;


/* Creates and compiles a pattern structure from the parameters, and
 * returns a pointer to it. To change the case sensitivity of a search
 * or the direction, the pattern must be recompiled. The search is for
 * a sequence of bytes; the pattern can contain embedded \0 characters.
 * Returns NULL if memory couldn't be allocated, 'pattern_length' is
 * zero, or 'pattern' does not point to a valid block of memory. */

Misc_TBMKpattern Misc_TBMKpattern_alloc(const unsigned char *pattern, unsigned int pattern_length, unsigned char reverse, unsigned char nocases);


/* Destroys a pattern structure created with Misc_TBMKpattern_alloc().
 * Note that a pointer to the value returned by that function is passed
 * as the paramter. On return, the value pointed to by 'pattern' has
 * been set to NULL. This function does nothing if 'pattern' is not a
 * valid address, or does not point to a valid Misc_TBMKpattern. */

void Misc_TBMKpattern_free(Misc_TBMKpattern *pattern);


/* Searches a block of memory for a compiled pattern. The parameter
 * 'text' points to the first byte that might possibly be examined in
 * the search. The search encompasses at most 'text_extent' bytes from
 * (and including) the character pointed to by 'text'; positive for
 * forward searching, negative for reverse searches. If 'all_matches'
 * is true, the total number of matches is returned; otherwise, the
 * search stops at the first match. Returns the number of non-overlapping
 * matches found (zero if no matches found) and the offset from 'text'
 * of the last match is returned by reference in 'match_position' (for
 * reverse searches, 'match_position' will be negative). Returns -1 if
 * 'pattern' is clearly not a valid pattern, 'text' does not point to a
 * valid block of memory of extent 'text_extent', the sign of 'text_extent'
 * is not valid for the type of search specified when the pattern was
 * compiled, or 'match_position' is not a valid address. */
int Misc_TBMKsearch(const Misc_TBMKpattern pattern, const unsigned char *text, int text_extent, unsigned char all_matches, int *match_position);

#endif /* _MISC_TBMK_H */
