/***************************************************************************
 * CLASS:		MiscImageView
 *
 *	For more information on MiscImageView, see the header file. 
 *
 * This object is included in the MiscKit by permission from the author
 * and its use is governed by the MiscKit license, found in the file
 * "LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
 * for a list of all applicable permissions and restrictions.
 ***************************************************************************/

#import <dpsclient/psops.h>
#import "MiscImageView.h"


@implementation MiscImageView

- initFrame: (const NXRect *)frameRect
{
    [super initFrame: frameRect];
	ivImage = nil;
	[self setBackgroundColor: NX_COLORLTGRAY];
	[self setDrawBackground: YES];
	[self setBorderType: NX_BEZEL];
	
    return self;
}

	
- free
{
	[ivImage free];
	return [super free];
}


// This likely is not the best way to implement this, because it attempts
// to deal with whether the image is maintained (retainData) and what
// dragImage is set to, but this way it works for both ImageView and when
// you start subclassing and using dragImage

- setImage: (NXImage *)anImage
{		
	if (dragImage != nil)
		if ([self retainData])
			dragImage = nil;
		else
		{
			ivImage = dragImage;
			dragImage = nil;
		 }		 

	if ([self image])
		[ivImage free];
    ivImage = anImage;
	
	[ivImage getSize: &ivSize];
	[ivImage setScalable: YES];
	
	[self display];
    return self;
}


// Though the docs for NXImage say that initFromFile: does not need an
// absolute pathname, I would recommend it. I had problems with it not 
// finding images even when they were in the app wrapper

- setImageByFilename: (char *)aFilename
{
	[self setImage: [ [NXImage alloc] initFromFile: aFilename] ];
	return self;
}


- (NXImage *)image
{
	return ivImage;
}


- setBackgroundColor: (NXColor)someColor
{
    backgroundColor = someColor;
	[self display];
    return self;
}


- (NXColor)backgroundColor
{
	return backgroundColor;
}


- setDrawBackground: (BOOL)aBool
{
	drawBackground = aBool;
	return self;
}


- (BOOL)drawBackground
{
	return drawBackground;
}


// Border Type can be one of NX_BEZEL, NX_GROOVE, NX_LINE, or NX_NOBORDER,
// same as the appkit's Box.

- setBorderType: (int)aType
{
	border = aType;
	[self display];
	return self;
}


- (int)borderType
{
	return border;
}


// Draw the border, background color and the image if there is one.
// I tried to add the scaling code to sizeTo:: but it did not seem
// that it was called every time the view was resized (almost all the
// time, but not good enough). So to be safe for now I lumped it all together.

- drawSelf: (const NXRect *)rects :(int)rectCount;
{
  NXPoint  offset;
  NXRect  imageRect;

	if ([self drawBackground])  
		switch (border)
		{
	  	  case NX_GROOVE:
	  		NXDrawGroove (&bounds, rects);
			break;
	  	  case NX_BEZEL:
	   		NXDrawGrayBezel(&bounds, rects);
			break;
	  	  case NX_LINE:
	  		NXSetColor (NX_COLORBLACK);
			NXRectFill (&bounds);
			NXFrameRect (&bounds);
			break;
	  	  case NX_NOBORDER:
	  		NXSetColor (backgroundColor);
			NXRectFill (&bounds);
	  		break;
	 	 }
	 
	// set so the background color will not wipe the border
	
    imageRect.origin.x = bounds.origin.x + 2;
   	imageRect.origin.y = bounds.origin.y + 2;
    imageRect.size.width = bounds.size.width - 4;
    imageRect.size.height = bounds.size.height - 4;
	
	if ([self drawBackground])
	{
    	NXSetColor (backgroundColor);
    	NXRectFill (&imageRect);
	 }

	// if there is no image, then no reason to continue
	
	if ([self image] == nil)
		return self;
	
	// use different offsets depending on if image scaled or not
			
	if (ivSize.width > bounds.size.width ||
			ivSize.height > bounds.size.height)
	{  
	  double xscale, yscale, scale;

		// scale the image and keep aspect ratio
		
	 	xscale = bounds.size.width / ivSize.width;
		yscale = bounds.size.height / ivSize.height;
		scale = ((xscale < yscale) ? xscale : yscale);
		scaledSize.width = (ivSize.width * scale) - 2.0; 
		scaledSize.height = (ivSize.height * scale) - 2.0;
		
		offset.x = (bounds.size.width - scaledSize.width) / 2;
		offset.y = (bounds.size.height - scaledSize.height) / 2;
		
		if (xscale < yscale)		// so the image is not right on the edge
			offset.x+=2.0; 
		else
			offset.y+=1.0;

		scaledSize.width -= 3.0;	// so when we move the offset,..
		scaledSize.height -= 3.0;	// image will be basically centered
		
		[ivImage setSize: &scaledSize];
	 }
	else
	{ 
		// no scaling needed
		
		offset = imageRect.origin;
		offset.x += (imageRect.size.width - ivSize.width) / 2.0;
		offset.y += (imageRect.size.height - ivSize.height) / 2.0;
		[ivImage setSize: &ivSize];
	 }

	[ivImage composite: NX_SOVER toPoint: &offset];
    return self;
}


- read: (NXTypedStream *)stream
{
	[super read: stream];
	ivImage = NXReadObject (stream);
	NXReadSize (stream, &ivSize);
	NXReadSize (stream, &scaledSize);
	backgroundColor = NXReadColor (stream);
	NXReadTypes (stream, "i", &border);
	
	return self;
}	
	
	
- write: (NXTypedStream *)stream
{
	[super write: stream];
	NXWriteObject (stream, ivImage);
	NXWriteSize (stream, &ivSize);
	NXWriteSize (stream, &scaledSize);
	NXWriteColor (stream, backgroundColor);
	NXWriteTypes (stream, "i", &border);
	
	return self;
}

@end
